// some utilities

// returns the object, whether you supply an id (string) or an object itself
function getObj(id) {
	var obj;
	if(typeof(id) == "string") {
		obj = getById(id) ? getById(id) : null;
	}
	else if(typeof(id) == "object") {
		obj = id;
	}
	
	return obj;
}

// returns the object with the correspondent ID
function getById(id) {
	if(document.getElementById)
		return document.getElementById(id);
	else if(document.all)
		return document.all[id];
	else
		return false;
}


// returns the current version number from the Info.plist
function getCurrentVersion() {
	var localReq = new XMLHttpRequest(); 
	localReq.open("GET", "Info.plist", false); 
	localReq.send(null); 
	
	var infoPlist = localReq.responseText; 
	
	var versionRegEx = /<key>CFBundleVersion<\/key>\s*<string>([^<]*)<\/string>/;
	var versionRaw = infoPlist.match(versionRegEx);
	var thisVersion = versionRaw[1];
	
	// display this version on the backside of the Widget
	verspan = getObj('thisVersion');
	if(undefined == thisVersion) {
		verspan.innerHTML = '';
		alert("An error occurred while checking the local version. Assuming local version up-to-date.");
		return 0;
	}
	
	verspan.innerHTML = getBeautifulVersionnumber(thisVersion);
	return thisVersion;
}

// returns the version-number in a comparable form
	// we support version-numbering with up to 3 dots, e.g. 1.8.1.4
	// the last 2 steads support up to --6-- numbers, first 2 up to --3--
	// 1.8.1.4 becomes 1 008 000001 000004
function getComparableVersionnumber(ver) {
	var prts = ver ? ver.split(/\./) : new Array();
	var version = (prts[0] * 1000000 * 1000000 * 1000) +
						(prts[1] ? (prts[1] * 1000000 * 1000000) : 0) +
						(prts[2] ? (prts[2] * 1000000) : 0) +
						(prts[3] ? (prts[3] * 1) : 0);
	return version;
}

// returns a human readable versionnumber, from original version or comparable versionnumber
	// 1.8.060429 becomes 1.8.beta
	// 1008000001000000 becomes 1.8.1
function getBeautifulVersionnumber(ver) {
	
	// internal version number
	if(ver > 1000000) {
		var parts = new Array();
		
		parts[3] = ver % 1000000;
		ver = (ver - parts[3]) / 1000000;
		parts[2] = ver % 1000000;
		ver = (ver - parts[2]) / 1000000;
		parts[1] = ver % 1000;
		ver = (ver - parts[1]) / 1000;
		parts[0] = ver % 1000;
	}
	
	// raw number
	else {
		var parts = ver ? ver.split(/\./) : new Array();
	}
	
	// compose
	var version = (parts[3] > 0) ? ((parts[3] > 1000) ? 'beta' : parts[3]) : undefined;
	version = (parts[2] > 0) ? (((parts[2] > 1000) ? 'beta' : parts[2]) + (version ? ("." + version) : '')) : ((undefined == version) ? undefined : 0 + "." + version);
	version = (parts[1] > 0) ? (parts[1] + (version ? ("." + version) : '')) : ((undefined == version) ? undefined : "0." + version);
	version = (parts[0] > 0) ? (parts[0] + (version ? ("." + version) : '.0')) : ((undefined == version) ? undefined : "0." + version);
	
	//alert(parts[0] + "." + parts[1] + "." + parts[2] + "." + parts[3] + " -- " + version);
	return version;
}


function parseChannelName(channel) {
	var channelname = channel;
	if(channelname)
		channelname = channelname.replace(/\+/g, " ");
	
	return channelname;
}


function removeKeyFromHash(removeKey, array)
{
	var result = new Object();
	for (var key in array)
	{
		if (key != removeKey) result[key] = array[key];
	}
	return result;
}

function keysFromHash(hash)
{
	var keys = new Array();
	for (var key in hash)
	{
		keys.push(key);
	}
	return keys;
}

function indexOfObjectInArray(array, object)
{
	if("object" != typeof(array)) return -1;
	for (i = 0; i < array.length; i++)
	{
		if (array[i] == object) return i;
	}
	return -1;
}

function inArray(value, arr)
{
	if("object" != typeof(arr)) return false;
	
	for(var i = 0; i < arr.length; i++)
	{
		if(arr[i] == value) return true;
	}
	return false;
}

function selectValue(element, value)
{
	if(!element) return;
	
	for (i = 0; i < element.length; i++)
	{
		if (element.options[i].value == value)
		{
			element.selectedIndex = i;
			return;
		}
	}
}

function openDiagnosis()
{
	blendDialogOut();
	if(window.widget) widget.openApplication("com.apple.NetworkDiagnostics");
}

function openConsole()
{
	if(window.widget) widget.openApplication("com.apple.Console");
}

function localizedWeekday(date)
{
	var tage;
	if ("DE" == currentLocale()) tage = new Array("Sonntag","Montag","Dienstag","Mittwoch","Donnerstag","Freitag","Sonnabend");
	else tage = new Array("Sonntag","Montag","Dienstag","Mittwoch","Donnerstag","Freitag","Samstag");	
	return tage[date.getDay()];
}

function printableDate(date)
{
	return localizedWeekday(date) + ', ' + date.getDate() + '.' + (date.getMonth() + 1) + '.';
}

function currentLocale()
{
	var locale = FernsehPlugin.currentLocale();
	if (undefined == locale) return "DE";
	var comps = locale.split("_");
	if (comps.length != 2) return "DE";
	return comps[1];
}

function mergeHashes(hashOne, hashTwo)
{
	var result = new Object();
	for (var key in hashOne)
	{
		result[key] = hashOne[key];
	}
	for (var key in hashTwo)
	{
		result[key] = hashTwo[key];
	}
	return result;
}

// 
// %E9%E8%E1%E0
//  %u0301

function deunicodifyLatinUmlauts(string)
{
	var result = escape (string);
	result = result.replace(/a%u0308/g, "\u00E4"); // ae
	result = result.replace(/A%u0308/g, "\u00C4"); // ae
	result = result.replace(/o%u0308/g, "\u00F6"); // oe
	result = result.replace(/O%u0308/g, "\u00D6"); // oe
	result = result.replace(/u%u0308/g, "\u00FC"); // ue
	result = result.replace(/U%u0308/g, "\u00DC"); // ue
	result = result.replace(/e%u0301/g, "\u00E9"); // e aigu
	result = result.replace(/a%u0301/g, "\u00E1"); // a aigu
	result = result.replace(/o%u0301/g, "\u00F3"); // o aigu
	result = result.replace(/e%u0300/g, "\u00E8"); // e grave
	result = result.replace(/a%u0300/g, "\u00E0"); // a grave
	result = result.replace(/o%u0300/g, "\u00F2"); // o grave
	result = unescape(result);
	return result;
}

function unicodifyLatinUmlauts(string)
{
	var result = escape (string);
	result = result.replace(/%E4/g, "a%u0308"); // ae
	result = result.replace(/%C4/g, "A%u0308"); // ae
	result = result.replace(/%F6/g, "o%u0308"); // oe
	result = result.replace(/%D6/g, "O%u0308"); // oe
	result = result.replace(/%FC/g, "u%u0308"); // ue
	result = result.replace(/%DC/g, "U%u0308"); // ue
	result = result.replace(/%E9/g, "e%u0301"); // e aigu
	result = result.replace(/%E1/g, "a%u0301"); // e aigu
	result = result.replace(/%F3/g, "o%u0301"); // o aigu
	result = result.replace(/%E8/g, "e%u0300"); // e grave
	result = result.replace(/%E0/g, "a%u0300"); // e grave
	result = result.replace(/%F2/g, "o%u0300"); // o grave
	result = unescape(result);
	return result;
}

// returns a SHALLOW copy of a hash
// not appropriate for DEEP copies!
function copyHash(hash)
{
	var result = new Object();
	for (var key in hash)
	{
		result[key] = hash[key];
	}
	return result;
}

function copyArray(array)
{
	var result = new Array();
	for (var i = 0; i < array.length; i++)
	{
		result.push(array[i]);
	}
	return result;
}

function debug(string)
{
	if (!debugMode) return;
	alert(string);
}


function has1043Features() {
	var version = navigator.appVersion;
	var regExp = /.+AppleWebKit\/([\d\.]+).+/;
	var result = regExp.exec(version);
	
	return (parseFloat(result[1]) >= 416.11);
}


function goToHomepage() {
	if(window.widget) {
		widget.openURL('http://widgetschmie.de/cgi-bin/index.pl?sub=widgets/Fernsehen');
	}
}


// JS serialization

function Serialize(obj) {
	var strSource = "";

	if (typeof(obj) == "undefined") {
		strSource = "undefined";
	}

	else if (obj == null) {
		strSource = "null";
	}

	else if (typeof(obj) == "boolean" || typeof(obj) == "number" || typeof(obj) == "function") {
		strSource = obj;
	}

	else if (typeof(obj) == "string") {
		strSource = "'" + obj.replace(/([\\'"])/g, "\\$1") + "'"; 		//" (just to avoid the texteditors problems with the single Notation Mark)
	}

	else if (typeof(obj) == "object") {
		if (obj.constructor == Boolean) {
			strSource = "new Boolean(" + obj.valueOf() + ")";
		}

		else if (obj.constructor == Number) {
			strSource = "new Number(" + obj.valueOf() + ")";
		}

		else if (obj.constructor == String) {
			strSource = "new String(" + Serialize(obj.toString()) + ")";
		}

		else if (obj.constructor == Date) {
			strSource = "new Date(" + obj.valueOf() + ")";
		}

		else if (obj.constructor == Array) {
			strSource = "[";
			for (var i=0; i<obj.length; i++)
				strSource += Serialize(obj[i]) + ",";
			if (i>0)
				strSource = strSource.substring(0, strSource.length-1);
			strSource += "]";
		}

		else if (obj.constructor == RegExp) {
			strSource = obj.toString();
		}

		else if (obj.constructor == Object) {
			var strSource = "{"
			for (var key in obj)
				strSource = strSource + "" + key + ":" + Serialize(obj[key]) + ","
			if (strSource.length > 1)
				strSource = strSource.substring(0, strSource.length-1);
			strSource += "}";
		}
	}

	else {
		throw new Error(0, "Serialize encountered unexpected object type: " + typeof(obj));
	}

	return strSource;
}


function Deserialize(strSource) {
		if (typeof strSource != "string")
				throw new Error(0, "Deserialize expects a string argument");

		var x;
		try {
			return eval("x = " + strSource);
		}
		catch(exc) {
			alert("Deserialize Error: " + exc);
		}
		return null;
}

